==> apache-samza-0.10.1-src.tgz: apache-samza-0.10.1-src/samza-kafka/src/test/scala/org/apache/samza/system/kafka/TestKafkaSystemProducer.scala <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.samza.system.kafka

import org.apache.samza.system.{OutgoingMessageEnvelope, SystemStream}
import org.junit.Test

import org.apache.kafka.clients.producer._
import java.util
import org.junit.Assert._
import org.scalatest.Assertions.intercept
import org.apache.kafka.common.errors.{TimeoutException, RecordTooLargeException}
import org.apache.samza.SamzaException


class TestKafkaSystemProducer {

  val someMessage = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "test".getBytes)
  val StreamNameNullOrEmptyErrorMsg = "Stream Name should be specified in the stream configuration file.";

  @Test
  def testKafkaProducer {
    val systemProducer = new KafkaSystemProducer(systemName = "test",
                                           getProducer = () => { new MockProducer(true) },
                                           metrics = new KafkaSystemProducerMetrics)
    systemProducer.register("test")
    systemProducer.start
    systemProducer.send("test", someMessage)
    assertEquals(1, systemProducer.producer.asInstanceOf[MockProducer].history().size())
    systemProducer.stop
  }

  @Test
  def testKafkaProducerUsingMockKafkaProducer {
    val mockProducer = new MockKafkaProducer(1, "test", 1)
    val systemProducer = new KafkaSystemProducer(systemName = "test",
                                                 getProducer = () => mockProducer,
                                                 metrics = new KafkaSystemProducerMetrics)
    systemProducer.register("test")
    systemProducer.start()
    systemProducer.send("test", someMessage)
    assertEquals(1, mockProducer.getMsgsSent)
    systemProducer.stop()
  }

  @Test
  def testKafkaProducerBufferedSend {
    val msg1 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "a".getBytes)
    val msg2 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "b".getBytes)
    val msg3 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "c".getBytes)

    val mockProducer = new MockKafkaProducer(1, "test", 1)
    val producerMetrics = new KafkaSystemProducerMetrics
    val systemProducer = new KafkaSystemProducer(systemName = "test",
                                                 getProducer = () => mockProducer,
                                                 metrics = producerMetrics)
    systemProducer.register("test")
    systemProducer.start()
    systemProducer.send("test", msg1)

    mockProducer.setShouldBuffer(true)
    systemProducer.send("test", msg2)
    systemProducer.send("test", msg3)
    assertEquals(1, mockProducer.getMsgsSent)

    val sendThread: Thread = mockProducer.startDelayedSendThread(2000)
    sendThread.join()

    assertEquals(3, mockProducer.getMsgsSent)
    assertEquals(0, producerMetrics.retries.getCount)
    systemProducer.stop()
  }

  @Test
  def testKafkaProducerFlushSuccessful {
    val msg1 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "a".getBytes)
    val msg2 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "b".getBytes)
    val msg3 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "c".getBytes)

    val mockProducer = new MockKafkaProducer(1, "test", 1)
    val systemProducer = new KafkaSystemProducer(systemName = "test",
                                                 getProducer = () => mockProducer,
                                                 metrics = new KafkaSystemProducerMetrics)
    systemProducer.register("test")
    systemProducer.start()
    systemProducer.send("test", msg1)

    mockProducer.setShouldBuffer(true)
    systemProducer.send("test", msg2)
    systemProducer.send("test", msg3)
    assertEquals(1, mockProducer.getMsgsSent)
    mockProducer.startDelayedSendThread(2000)
    systemProducer.flush("test")
    assertEquals(3, mockProducer.getMsgsSent)
    systemProducer.stop()
  }

  @Test
  def testKafkaProducerFlushWithException {
    val msg1 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "a".getBytes)
    val msg2 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "b".getBytes)
    val msg3 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "c".getBytes)
    val msg4 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "d".getBytes)

    val mockProducer = new MockKafkaProducer(1, "test", 1)
    val systemProducer = new KafkaSystemProducer(systemName = "test",
                                                 getProducer = () => mockProducer,
                                                 metrics = new KafkaSystemProducerMetrics())
    systemProducer.register("test")
    systemProducer.start()
    systemProducer.send("test", msg1)

    mockProducer.setShouldBuffer(true)
    systemProducer.send("test", msg2)
    mockProducer.setErrorNext(true, new RecordTooLargeException())
    systemProducer.send("test", msg3)
    systemProducer.send("test", msg4)

    assertEquals(1, mockProducer.getMsgsSent)

    mockProducer.startDelayedSendThread(2000)
    val thrown = intercept[SamzaException] {
      systemProducer.flush("test")
    }
    assertTrue(thrown.isInstanceOf[SamzaException])
    assertEquals(2, mockProducer.getMsgsSent)
    systemProducer.stop()
  }

  @Test
  def testKafkaProducerWithRetriableException {
    val msg1 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "a".getBytes)
    val msg2 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "b".getBytes)
    val msg3 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "c".getBytes)
    val msg4 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "d".getBytes)
    val numMaxRetries = 3

    val mockProducer = new MockKafkaProducer(1, "test", 1)
    val producerMetrics = new KafkaSystemProducerMetrics()
    val producer = new KafkaSystemProducer(systemName =  "test",
      getProducer = () => mockProducer,
      metrics = producerMetrics,
      maxRetries = numMaxRetries)

    producer.register("test")
    producer.start()
    producer.send("test", msg1)
    producer.send("test", msg2)
    producer.send("test", msg3)
    producer.flush("test")

    assertEquals(0, producerMetrics.retries.getCount)
    mockProducer.setErrorNext(true, new TimeoutException())

    val thrown = intercept[SamzaException] {
      producer.send("test", msg4)
    }
    assertTrue(thrown.isInstanceOf[SamzaException])
    assertTrue(thrown.getCause.isInstanceOf[TimeoutException])
    assertEquals(true, producer.sendFailed.get())
    assertEquals(3, mockProducer.getMsgsSent)
    assertEquals(numMaxRetries, producerMetrics.retries.getCount)
    producer.stop()
  }

  @Test
  def testKafkaProducerWithNonRetriableExceptions {
    val msg1 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "a".getBytes)
    val msg2 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "b".getBytes)
    val msg3 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "c".getBytes)
    val msg4 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "d".getBytes)
    val producerMetrics = new KafkaSystemProducerMetrics()

    val mockProducer = new MockKafkaProducer(1, "test", 1)
    val producer = new KafkaSystemProducer(systemName =  "test",
                                           getProducer = () => mockProducer,
                                           metrics = producerMetrics)
    producer.register("test")
    producer.start()
    producer.send("test", msg1)
    producer.send("test", msg2)
    producer.send("test", msg3)
    mockProducer.setErrorNext(true, new RecordTooLargeException())

    val thrown = intercept[SamzaException] {
       producer.send("test", msg4)
    }
    assertTrue(thrown.isInstanceOf[SamzaException])
    assertTrue(thrown.getCause.isInstanceOf[RecordTooLargeException])
    assertEquals(true, producer.sendFailed.get())
    assertEquals(3, mockProducer.getMsgsSent)
    assertEquals(0, producerMetrics.retries.getCount)
    producer.stop()
  }

  @Test
  def testKafkaProducerFlushMsgsWhenStop {
    val msg1 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "a".getBytes)
    val msg2 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "b".getBytes)
    val msg3 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "c".getBytes)
    val msg4 = new OutgoingMessageEnvelope(new SystemStream("test2", "test"), "d".getBytes)

    val mockProducer = new MockKafkaProducer(1, "test", 1)
    val systemProducer = new KafkaSystemProducer(systemName = "test",
                                                 getProducer = () => mockProducer,
                                                 metrics = new KafkaSystemProducerMetrics)
    systemProducer.register("test")
    systemProducer.register("test2")
    systemProducer.start()
    systemProducer.send("test", msg1)

    mockProducer.setShouldBuffer(true)
    systemProducer.send("test", msg2)
    systemProducer.send("test", msg3)
    systemProducer.send("test2", msg4)
    assertEquals(1, mockProducer.getMsgsSent)
    mockProducer.startDelayedSendThread(2000)
    assertEquals(1, mockProducer.getMsgsSent)
    systemProducer.stop()
    assertEquals(4, mockProducer.getMsgsSent)
  }

  @Test
  def testSystemStreamNameNullOrEmpty {
    val omeStreamNameNull = new OutgoingMessageEnvelope(new SystemStream("test", null), "a".getBytes)
    val omeStreamNameEmpty = new OutgoingMessageEnvelope(new SystemStream("test", ""), "a".getBytes)
    val mockProducer = new MockKafkaProducer(1, "testMock", 1)
    val producer = new KafkaSystemProducer(systemName = "test", getProducer = () => mockProducer,
                                           metrics = new KafkaSystemProducerMetrics)

    val thrownNull = intercept[IllegalArgumentException] {
      producer.register("test1")
      producer.start()
      producer.send("testSrc1", omeStreamNameNull)
      assertEquals(0, mockProducer.getMsgsSent)
    }
    val thrownEmpty = intercept[IllegalArgumentException] {
      producer.register("test2")
      producer.start()
      producer.send("testSrc2", omeStreamNameEmpty)
      assertEquals(0, mockProducer.getMsgsSent)
    }
    assertTrue(thrownNull.getMessage() == StreamNameNullOrEmptyErrorMsg)
    assertTrue(thrownEmpty.getMessage() == StreamNameNullOrEmptyErrorMsg)
  }
}

==> apache-samza-0.12.0-src.tgz: apache-samza-0.12.0-src/samza-kafka/src/test/scala/org/apache/samza/system/kafka/TestKafkaSystemProducer.scala <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.samza.system.kafka

import org.apache.samza.system.{OutgoingMessageEnvelope, SystemStream}
import org.junit.Test

import org.apache.kafka.clients.producer._
import java.util
import org.junit.Assert._
import org.scalatest.Assertions.intercept
import org.apache.kafka.common.errors.{TimeoutException, RecordTooLargeException}
import org.apache.kafka.test.MockSerializer
import org.apache.samza.SamzaException


class TestKafkaSystemProducer {

  val someMessage = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "test".getBytes)
  val StreamNameNullOrEmptyErrorMsg = "Stream Name should be specified in the stream configuration file.";

  @Test
  def testKafkaProducer {
    val mockProducer = new MockProducer(true, new MockSerializer, new MockSerializer)
    val systemProducer = new KafkaSystemProducer(systemName = "test",
                                                 getProducer = () => mockProducer,
                                                 metrics = new KafkaSystemProducerMetrics)
    systemProducer.register("test")
    systemProducer.start
    systemProducer.send("test", someMessage)
    assertEquals(1, systemProducer.producer.asInstanceOf[MockProducer[Array[Byte], Array[Byte]]].history().size())
    systemProducer.stop
  }

  @Test
  def testKafkaProducerUsingMockKafkaProducer {
    val mockProducer = new MockKafkaProducer(1, "test", 1)
    val systemProducer = new KafkaSystemProducer(systemName = "test",
                                                 getProducer = () => mockProducer,
                                                 metrics = new KafkaSystemProducerMetrics)
    systemProducer.register("test")
    systemProducer.start()
    systemProducer.send("test", someMessage)
    assertEquals(1, mockProducer.getMsgsSent)
    systemProducer.stop()
  }

  @Test
  def testKafkaProducerBufferedSend {
    val msg1 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "a".getBytes)
    val msg2 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "b".getBytes)
    val msg3 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "c".getBytes)

    val mockProducer = new MockKafkaProducer(1, "test", 1)
    val producerMetrics = new KafkaSystemProducerMetrics
    val systemProducer = new KafkaSystemProducer(systemName = "test",
                                                 getProducer = () => mockProducer,
                                                 metrics = producerMetrics)
    systemProducer.register("test")
    systemProducer.start()
    systemProducer.send("test", msg1)

    mockProducer.setShouldBuffer(true)
    systemProducer.send("test", msg2)
    systemProducer.send("test", msg3)
    assertEquals(1, mockProducer.getMsgsSent)

    val sendThread: Thread = mockProducer.startDelayedSendThread(2000)
    sendThread.join()

    assertEquals(3, mockProducer.getMsgsSent)
    systemProducer.stop()
  }

  @Test
  def testKafkaProducerFlushSuccessful {
    val msg1 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "a".getBytes)
    val msg2 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "b".getBytes)
    val msg3 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "c".getBytes)

    val mockProducer = new MockKafkaProducer(1, "test", 1)
    val systemProducer = new KafkaSystemProducer(systemName = "test",
                                                 getProducer = () => mockProducer,
                                                 metrics = new KafkaSystemProducerMetrics)
    systemProducer.register("test")
    systemProducer.start()
    systemProducer.send("test", msg1)

    mockProducer.setShouldBuffer(true)
    systemProducer.send("test", msg2)
    systemProducer.send("test", msg3)
    assertEquals(1, mockProducer.getMsgsSent)
    mockProducer.startDelayedSendThread(2000)
    systemProducer.flush("test")
    assertEquals(3, mockProducer.getMsgsSent)
    systemProducer.stop()
  }

  @Test
  def testKafkaProducerFlushWithException {
    val msg1 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "a".getBytes)
    val msg2 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "b".getBytes)
    val msg3 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "c".getBytes)
    val msg4 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "d".getBytes)

    val mockProducer = new MockKafkaProducer(1, "test", 1)
    val systemProducer = new KafkaSystemProducer(systemName = "test",
                                                 getProducer = () => mockProducer,
                                                 metrics = new KafkaSystemProducerMetrics())
    systemProducer.register("test")
    systemProducer.start()
    systemProducer.send("test", msg1)

    mockProducer.setShouldBuffer(true)
    systemProducer.send("test", msg2)
    mockProducer.setErrorNext(true, new RecordTooLargeException())
    systemProducer.send("test", msg3)
    systemProducer.send("test", msg4)

    assertEquals(1, mockProducer.getMsgsSent)

    mockProducer.startDelayedSendThread(2000)
    val thrown = intercept[SamzaException] {
      systemProducer.flush("test")
    }
    assertTrue(thrown.isInstanceOf[SamzaException])
    assertEquals(3, mockProducer.getMsgsSent) // msg1, msg2 and msg4 will be sent
    systemProducer.stop()
  }

  @Test
  def testKafkaProducerWithRetriableException {
    val msg1 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "a".getBytes)
    val msg2 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "b".getBytes)
    val msg3 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "c".getBytes)
    val msg4 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "d".getBytes)

    val mockProducer = new MockKafkaProducer(1, "test", 1)
    val producerMetrics = new KafkaSystemProducerMetrics()
    val producer = new KafkaSystemProducer(systemName =  "test",
      getProducer = () => mockProducer,
      metrics = producerMetrics)

    producer.register("test")
    producer.start()
    producer.send("test", msg1)
    producer.send("test", msg2)
    producer.send("test", msg3)
    producer.flush("test")

    mockProducer.setErrorNext(true, new TimeoutException())

    producer.send("test", msg4)
    val thrown = intercept[SamzaException] {
      producer.flush("test")
    }
    assertTrue(thrown.isInstanceOf[SamzaException])
    assertTrue(thrown.getCause.isInstanceOf[TimeoutException])
    assertEquals(3, mockProducer.getMsgsSent)
    producer.stop()
  }

  @Test
  def testKafkaProducerWithNonRetriableExceptions {
    val msg1 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "a".getBytes)
    val msg2 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "b".getBytes)
    val msg3 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "c".getBytes)
    val msg4 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "d".getBytes)
    val producerMetrics = new KafkaSystemProducerMetrics()

    val mockProducer = new MockKafkaProducer(1, "test", 1)
    val producer = new KafkaSystemProducer(systemName =  "test",
                                           getProducer = () => mockProducer,
                                           metrics = producerMetrics)
    producer.register("test")
    producer.start()
    producer.send("test", msg1)
    producer.send("test", msg2)
    producer.send("test", msg3)
    mockProducer.setErrorNext(true, new RecordTooLargeException())

    producer.send("test", msg4)
    val thrown = intercept[SamzaException] {
       producer.flush("test")
    }
    assertTrue(thrown.isInstanceOf[SamzaException])
    assertTrue(thrown.getCause.isInstanceOf[RecordTooLargeException])
    assertEquals(3, mockProducer.getMsgsSent)
    producer.stop()
  }

  @Test
  def testKafkaProducerFlushMsgsWhenStop {
    val msg1 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "a".getBytes)
    val msg2 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "b".getBytes)
    val msg3 = new OutgoingMessageEnvelope(new SystemStream("test", "test"), "c".getBytes)
    val msg4 = new OutgoingMessageEnvelope(new SystemStream("test2", "test"), "d".getBytes)

    val mockProducer = new MockKafkaProducer(1, "test", 1)
    val systemProducer = new KafkaSystemProducer(systemName = "test",
                                                 getProducer = () => mockProducer,
                                                 metrics = new KafkaSystemProducerMetrics)
    systemProducer.register("test")
    systemProducer.register("test2")
    systemProducer.start()
    systemProducer.send("test", msg1)

    mockProducer.setShouldBuffer(true)
    systemProducer.send("test", msg2)
    systemProducer.send("test", msg3)
    systemProducer.send("test2", msg4)
    assertEquals(1, mockProducer.getMsgsSent)
    mockProducer.startDelayedSendThread(2000)
    assertEquals(1, mockProducer.getMsgsSent)
    systemProducer.stop()
    assertEquals(4, mockProducer.getMsgsSent)
  }

  @Test
  def testSystemStreamNameNullOrEmpty {
    val omeStreamNameNull = new OutgoingMessageEnvelope(new SystemStream("test", null), "a".getBytes)
    val omeStreamNameEmpty = new OutgoingMessageEnvelope(new SystemStream("test", ""), "a".getBytes)
    val mockProducer = new MockKafkaProducer(1, "testMock", 1)
    val producer = new KafkaSystemProducer(systemName = "test", getProducer = () => mockProducer,
                                           metrics = new KafkaSystemProducerMetrics)

    val thrownNull = intercept[IllegalArgumentException] {
      producer.register("test1")
      producer.start()
      producer.send("testSrc1", omeStreamNameNull)
      assertEquals(0, mockProducer.getMsgsSent)
    }
    val thrownEmpty = intercept[IllegalArgumentException] {
      producer.register("test2")
      producer.start()
      producer.send("testSrc2", omeStreamNameEmpty)
      assertEquals(0, mockProducer.getMsgsSent)
    }
    assertTrue(thrownNull.getMessage() == StreamNameNullOrEmptyErrorMsg)
    assertTrue(thrownEmpty.getMessage() == StreamNameNullOrEmptyErrorMsg)
  }
}
